#
# The Channel Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

#------------------------------------------------------------------------


# $Id: slashbox.py,v 1.2 2000/07/08 21:34:46 richard Exp $
# Copyright (c) 2000 Richard Offer. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
# Richard Offer BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
# Except as contained in this notice, the name of Richard Offer shall not be
# used in advertising or otherwise to promote the sale, use or other dealings
# in this Software without prior written authorization from Richard Offer.

#========================================================================

import stat
import xmllib
import urllib
import urlparse
import os
import time
import string
import sys
from xml.sax import saxlib,saxexts

import App, Globals, OFS

from Persistence import Persistent
from Globals import HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from slashbox import slashbox


def manage_addChannel(self, id, title, url, REQUEST={}):
	""" Add a new RSS Channel """

	obj=Channel(id=id, url=url, title=title)
	self._setObject(id, obj)
	obj=getattr(self, id)

	if hasattr(obj, 'postInitialisation'):
		obj.postInitialisation()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''	

manage_addChannelForm=HTMLFile('manage_addChannelForm', globals())

class Channel(Folder, Persistent, Implicit):
	""" An RSS Channel -- Zope Style """

	# showChannel=HTMLFile('showChannel',globals())

	manage_editChannelForm=HTMLFile('manage_editChannelForm',globals())
	about=HTMLFile('about', globals())
	lastFetched=0
	meta_type='Channel'
	disabled=0
	lastStatus=0
	icon='misc_/ChannelTag/Channel.gif'
	manage_options=(
		{'label':'Manage Channel', 'action':'manage_editChannelForm'},
		{'label':'Contents', 'action':'manage_main'},
		{'label':'View', 'action':'index_html'},
		{'label':'About', 'action':'about'},
		)

	__ac_permission__=(
		('Manage Channels', ('manage_editChannelForm',
							  'manage_editChannel',)),
		)
	
	def __init__(self, id, url, title=''):
		self.id=id
		# self.url=urlparse.urlparse(url)
		self.url=url
		self.title=title
		self.setName(title)

		_u=urlparse.urlparse(self.url)
		self.site= _u[0]+ "://" + _u[1] + "/"
		# self.name= _u[1]
		
	def postInitialisation(self):
		self.fetchRSS()

	def stories(self):
		if hasattr(self, 'listsize'):
			listsize=getattr(self, 'listsize')
			if listsize:
				return self.dh.stories()[:listsize]
			else:
				return self.dh.stories()
		return self.dh.stories()
	
	def fetchRSS(self):
		(fname, headers)=urllib.urlretrieve(self.url)
		
		fp=open(fname)
		xmlp=saxexts.make_parser()

		self.dh=_myrdf()
		xmlp.setDocumentHandler(self.dh)
		xmlp.parseFile(fp)

		self.lastFetched=time.time()
		self._p_changed=1

	def setName(self, name):
		self.name=name
		
	def index_html(self, REQUEST={}):
		""" The Index File """
		if not self.disabled:
			if (self.lastFetched + int(self.freq)) < time.time():
				try:
					self.fetchRSS()
					self.lastFetched=time.time()
					self.lastStatus=1
					self.disabled=0
				except:
					self.lastStatus=0
					self.lastFail=self.lastFail+1
					if self.lastFail == 3:
						self.disabled=1

			return self.showChannel(self, REQUEST)
		
	def manage_editChannel(self, url, title, enable=0, REQUEST={}):
		""" Edit Channel Properties """
		self.url=url
		self.title=title
		self.name=title
		
		if enable:
			self.disable=0

		if REQUEST:
			return self.manage_main(self, REQUEST)
		return ''
		
class _myrdf(saxlib.HandlerBase):
	def __init__(self):
		self._in={}
		self._in["title"]=0
		self._in["link"]=0
		self._in["description"]=0
		self._txt={}
		self._txt["title"]=""
		self._txt["link"]=""
		self._txt["description"]=""

		self._stories=[]


	def startElement(self,ele,attr):
		if ele == 'item' :
			self._txt={}
			self._txt["title"]=""
			self._txt["link"]=""
			self._txt["description"]=""
		elif self._in.has_key(ele) :
			self._in[ele]=1
			
		else:
			pass

	def endElement(self,ele):
		if self._in.has_key(ele) :
			self._in[ele] = 0
			self._txt[ele] = string.join(string.split((self._txt[ele])))
		elif ele == 'item'  :
			self._stories.append(self._txt)

			
	
	def characters(self,ch,start,length):
		for i in self._in.keys():
				if self._in[i] == 1:
					self._txt[i] = self._txt[i] + ch[start:start+length]

	def stories(self):
		return self._stories
