#
# The Channel Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from Persistence import Persistent
from Globals import HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from urllib import quote

def manage_addChannelManagement(self, id, title='', REQUEST={}):
        """ Add a new Channel Management System """

        obj=ChannelManagement(id, title)
        self._setObject(id, obj)
        obj=getattr(self, id)
        if hasattr(obj, 'postInitialisation'):
                obj.postInitialisation()

        if REQUEST:
                return self.manage_main(self, REQUEST)
        return ''

manage_addChannelManagementForm=HTMLFile('manage_addChannelManagementForm', globals())

from Channel import Channel, manage_addChannel, manage_addChannelForm

class ChannelManagement(Folder, Persistent, Implicit):
	""" Manage your RSS Channels """

	meta_type="Channel Management"

	icon='ChannelManagement.gif'
	
	manage_addChannelForm = manage_addChannelForm
	manage_addChannel     = manage_addChannel
	manage_addDTMLMethod=OFS.DTMLMethod.addDTMLMethod

	manage_configureChannelManagementForm=HTMLFile('manage_configureChannelManagementForm', globals())
	
	about=HTMLFile('about', globals())
	
	manage_options=(
		{'label':'Contents', 'action':'manage_main'},
		{'label':'New Channel', 'action':'manage_addChannelForm'},
		{'label':'Configuration', 'action':'manage_configureChannelManagementForm'},
		{'label':'About', 'action':'about'},
		)

	__ac_permission__=(
		('Add Channels', ('manage_addChannelForm', 'manage_addChannel',)),
		('Manage Channels', ('manage_configureChannelManagementForm',
							  'manage_configureChannelManagement',)),
		)

	pre_canned_channels=[
		{'id':'Zope', 'url':'http://www.zope.org/SiteIndex/news.rss', 'name':'Zope'},
		{'id':'Slashdot', 'url':'http://slashdot.org/slashdot.rdf', 'name':'Slashdot'},
		{'id':'Freshmeat', 'url':'http://freshmeat.net/backend/fm.rdf', 'name':'Freshmeat'},
		{'id':'Parnassus', 'url':'http://www.vex.net/parnassus/parnassus.rss', 'name':'Vaults of Parnassus'},
		{'id':'SourceForge', 'url':'http://sourceforge.net/export/rss_sfnewreleases.php', 'name':'SourceForge New Releases'},
		{'id':'SourceForgeNews', 'url':'http://sourceforge.net/export/rss_sfnews.php', 'name':'SourceForge News'},
		{'id':'DailyDaemonNews','url':'http://daily.daemonnews.org/ddn.rdf.php3 ', 'name':'Daily Daemon News'},
		]

	listsize = 0
	freq = 600 # seconds

	
	def __init__(self, id, title=''):
		self.id=id
		self.title=title

	def postInitialisation(self):
		self.base_url=self.absolute_url()
		self.manage_addDTMLMethod(id='showChannel', title='Channel Summary Box', file=HTMLFile('showChannel', globals()))

		# Just for fun add some pre-canned channels
		for c in self.pre_canned_channels:
			try:
				print "Adding %s Feed"%(c['name'])
				self.manage_addChannel(id=c['id'],
									   title=c['name'],
									   url=c['url'])
			except:
				print "Adding %s Failed."%(c['name'])
				pass
										

	# Get One Channel
	def getChannel(self, channelId):
		if hasattr(self, channelId):
			channel=getattr(self, channelId)
			return channel
		
	def channelList(self):
		""" Return the Channels """
		return self.objectIds('Channel')

	def displayChannel(self, channelId, REQUEST={}):
		""" Display One Channel """
		if hasattr(self, channelId):
			channel=getattr(self, channelId)
			return channel.index_html(REQUEST=REQUEST)


	def manage_configureChannelManagement(self, listsize, freq, REQUEST={}):
		""" Configure Channel Stuff """

		listsize=int(listsize)
		if listsize < 0:
			listsize=0
		self.listsize=listsize

		ifreq=int(freq)
		if ifreq < 10:
			ifreq = 10

		self.freq=ifreq*60 # Go from minutes to seconds

		if REQUEST:
			return self.manage_main(self, REQUEST)
		return ''
		
